#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

from qubtree import Node
import QUB
from Dialogs import *
from math import *
from string import *
import re
import random
import CurveFitting
import FuzzyFit


# adapter from FuzzyFit.py to QUB curve fitting plugin system


def MissingName(msg): # extract the unknown name from a NameError message
    m = re.search("'([^']+)'", msg)
    if not m:
        raise NameError, msg
    return m.group(1)    


class FuzzyFitter(CurveFitting.NLFitter):
    def __init__(self):
        CurveFitting.NLFitter.__init__(self)
        self.fitter = FuzzyFit.FuzzyFitter()
        self.setEqn('Amp * exp( - x / Tau )')
        self.mag = 0.05
        self.nfuzz = 10
        self.maxnfuzz = 20
        self.decimation = 10

    def func(self, paramvals, x):
        try:
            return apply( self.fitter.func, paramvals + [x] )
        except:
            return 0.0

    def getEqn(self):
        return self.fitter.eqn
        
    def setNumComponents(self, n):
        CurveFitting.NLFitter.setNumComponents(self, n)
        self.fitter.setComponentCount(n)
        self.updateFitterParams()

    def useData(self, dataset):
        CurveFitting.NLFitter.useData(self, dataset)
        self.fitter.useData(self.X, self.Y, self.decimation)
        
    def promptOptions(self):
        valid = 0
        msg = 'Please enter an expression in Python style'
        neweqn = self.fitter.eqn
        
        while not valid:
            options = QUB.Dialog(msg,
                                 ['OK', 'Cancel'],
                                 [ DialogText('eqn', 'y(x) = ', neweqn, acceptString),
                                   DialogText('mag', 'Fuzz Magnitude:', self.mag, acceptFloatGreaterThan(0.0)),
                                   DialogText('nfuzz', 'Fuzz Samples:', self.nfuzz, acceptIntGreaterThan(0)),
                                   DialogText('maxnfuzz', 'Fuzz Max Samples:', self.maxnfuzz, acceptIntGreaterThan(0)),
                                   DialogText('decimation', 'Decimation (use every k points):', self.decimation, acceptIntGreaterThan(0)) ])
            if options['button'] == 'Cancel':
                return
            else:
                self.mag = options['mag']
                self.nfuzz = options['nfuzz']
                self.maxnfuzz = options['maxnfuzz']
                self.decimation = options['decimation']
                self.fitter.useData(self.X, self.Y, self.decimation)
                try:
                    neweqn = options['eqn']
                    if neweqn != self.fitter.eqn:
                        self.setEqn( neweqn )
                    valid = 1
                except Exception, x:
                    msg = str(x)

    def setEqn(self, neweqn):
        fitter = self.fitter
        self.fitter.setEqn(neweqn)
        initParams = [(fitter.paramName(i), fitter.param(0, i), -1.0e20, 1.0e20) for i in range(fitter.paramCount())]
        apply( self.initParams, [len(initParams)] + initParams )
        self.eqn = neweqn
        self.updateFitterParams()

    def updateFitterParams(self):
        fitter = self.fitter
        for c in range(fitter.componentCount()):
            for i in range(fitter.paramCount()):
                fitter.setParam(c, i, self.params[c][fitter.paramNames[i]].data[0])

    def fit(self, maxiter, precision=0.01):
        self.updateFitterParams()
        self.fitter.fit(self.mag, self.nfuzz, self.maxnfuzz, maxiter, precision)

        def setparam(i, vals):
            par = self.params[i].child
            for v in vals:
                par.data[0] = v
                par = par.sibling
        for c in range(self.getNumComponents()):
            setparam(c, self.fitter.pvals[c])
        self.updateParamLists()

        self.onFitDone()        


CurveFitting.fitters['Fuzzy'] = FuzzyFitter


class DumbGradFitter(FuzzyFitter):
    def __init__(self):
        CurveFitting.NLFitter.__init__(self)
        self.fitter = FuzzyFit.DumbGradFitter()
        self.setEqn('Amp * exp( - x / Tau )')
        self.rad = 0.5
        self.decay = 0.9
        self.ncircle = 100
        self.linmean = 1.0
        self.lincount = 100
        self.decimation = 10
    def promptOptions(self):
        valid = 0
        msg = 'Please enter an expression in Python style'
        neweqn = self.fitter.eqn
        
        while not valid:
            options = QUB.Dialog(msg,
                                 ['OK', 'Cancel'],
                                 [ DialogText('eqn', 'y(x) = ', neweqn, acceptString),
                                   DialogText('rad', 'Circle Radius:', self.rad, acceptFloatGreaterThan(0.0)),
                                   DialogText('decay', 'Circle Decay:', self.decay, acceptFloatBetween(0.0, 1.0)),
                                   DialogText('ncircle', 'Circle Samples:', self.ncircle, acceptIntGreaterThan(0)),
                                   DialogText('linmean', 'Line Search Mean:', self.linmean, acceptFloatGreaterThan(0.0)),
                                   DialogText('lincount', 'Line Samples:', self.lincount, acceptIntGreaterThan(0)),
                                   DialogText('decimation', 'Decimation (use every k points):', self.decimation, acceptIntGreaterThan(0)) ])
            if options['button'] == 'Cancel':
                return
            else:
                self.rad = options['rad']
                self.decay = options['decay']
                self.ncircle = options['ncircle']
                self.linmean = options['linmean']
                self.lincount = options['lincount']
                self.decimation = options['decimation']
                self.fitter.useData(self.X, self.Y, self.decimation)
                try:
                    neweqn = options['eqn']
                    if neweqn != self.fitter.eqn:
                        self.setEqn( neweqn )
                    valid = 1
                except Exception, x:
                    msg = str(x)
    def fit(self, maxiter, precision=0.01):
        self.updateFitterParams()
        self.fitter.fit(self.rad, self.decay, self.ncircle, self.linmean, self.lincount, maxiter, precision)

        def setparam(i, vals):
            par = self.params[i].child
            for v in vals:
                par.data[0] = v
                par = par.sibling
        for c in range(self.getNumComponents()):
            setparam(c, self.fitter.pvals[c])
        self.updateParamLists()

        self.onFitDone()        

# CurveFitting.fitters['DumbGrad'] = DumbGradFitter
